'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var PowerShell = _interopDefault(require('powershell'));
var PythonShell = _interopDefault(require('python-shell'));
var path = _interopDefault(require('path'));
var difference = _interopDefault(require('lodash.difference'));
var util = require('util');

function findAllApps(root) {
  const apps = [];

  let nodes = [root];
  while (nodes.length) {
    const node = nodes.shift();

    if (node.type === 'app') {
      apps.push(node);
    } else {
      if (node.children) {
        nodes.push(...node.children);
      }
    }
  }

  return apps;
}

const exec = async cmd => {
  return new Promise((resolve, reject) => {
    const process = new PowerShell(cmd);

    process.on('error', reject);

    process.on('output', data => {
      const pid = data.split('\r\n')[3].split(' ').filter(ch => ch.length)[5];

      resolve(pid);
    });
  });
};

var _extends$1 = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function resolvePython(file) {
  return path.join(__dirname, path.sep, file);
}

const defaultOptions = {
  pythonPath: 'python'
};

const PythonShellRunAsync = util.promisify(PythonShell.run);
async function runPython(file, options = {}) {
  return PythonShellRunAsync(resolvePython(file), _extends$1({}, defaultOptions, options));
}

const timeout = n => new Promise(resolve => setTimeout(resolve, n));

class WorkflowWmWindowsPython {
  async screen() {
    const [rect] = await runPython('get_desktop_rect.py');
    return JSON.parse(rect);
  }

  async apply(layout) {
    const apps = findAllApps(layout);

    const { startOnPositionByWindowClass, startOnPositionByReturnedPid } = this;

    const context = {
      platform: 'win32',
      wm: 'default',
      startOnPositionByWindowClass: startOnPositionByWindowClass.bind(this),
      startOnPositionByReturnedPid: startOnPositionByReturnedPid.bind(this)
    };

    for (let app of apps) {
      await app.open(app, context, app.children);
    }
  }

  async startOnPositionByReturnedPid({ cmd, args, position }) {
    const pid = await exec(`Start-Process ${cmd} -PassThru "${args.join(' ')}"`);
    const { left, top, width, height } = position;
    this.setPosition(pid, top, left, width, height);
  }

  async startOnPositionByWindowClass({ cmd, args, className, position }) {
    const before = await this.getListOfWindows(className);
    await exec(`Start-Process ${cmd} -PassThru "${args.join(' ')}"`);

    await timeout(1000);

    const after = await this.getListOfWindows(className);

    const windowIds = difference(after, before);
    const { left, top, width, height } = position;
    for (let windowId of windowIds) {
      this.setPositionByWindowId(windowId, left, top, width, height);
    }
  }

  async setPosition(pid, top, left, width, height) {
    const options = {
      args: [pid, left, top, width, height]
    };
    return await runPython('set_position.py', options);
  }

  async getListOfWindows(className) {
    const options = {
      args: [className]
    };

    const [windowIds] = await runPython('get_list_of_windows.py', options);

    return JSON.parse(windowIds);
  }

  async setPositionByWindowId(windowId, left, top, width, height) {
    const options = {
      args: [windowId, left, top, width, height]
    };

    await runPython('set_position_by_window_id.py', options);
  }
}

exports.WorkflowWmWindowsPython = WorkflowWmWindowsPython;
